##
## Author: Dave Kolb (dave@49thLatitude.com)
##
## Creates a full volume backup using Terabyte IFW named for the date and time of the backup
##
## TODO
## ----
## - fail if Outlook or OneNote is running
##

## set error handling
$ErrorActionPreference = "stop"
$WarningPreference = "stop"

################################# configure machine local values here #####################
## target folder location where the backups should be created
$BackupFolder = "D:\IFW"

## drive letter of the disk that should be backed up (no : or \ - just the 1 character drive letter)
$SourceDriveLetter = "C"

## Select "Full" or "Diff"
$BackupType = "Full"

## location where IFW executable was installed
$IFWPath = "${env:ProgramFiles(x86)}\TeraByte Unlimited"

## comment the below line if you do not want validation - /vb does a byte-for-byte validation
$validation = "/vb" ## this does byte for byte verification
##$validation = "/v" ## this does verification but not byte for byte!

## logging - turn on full logging
$logging = "/logl:10"

##locking - VSS or PHYLOCK
##$locking = "/pldis /usevss"  ## VSS locking
$locking = "/plrell" ## PHYLOCK locking

## if you want an encrypted backup file, uncomment the below line
## and change xyzzy to the password you want and set the encryption level
## /enc:1 is password protected but no encryption and /enc:3 is AES 256 encryption
##$encryption = "/pw:""xyzzy"" /enc:3"

## uncomment the below line if you want compression
## 0 is no compression and 1 is standard compression - see user manual for other levels
##$compression = "/comp:1"
###########################################################################################

## function to convert from drive letter to drive index
function Get-DriveIndex
{
	[CmdletBinding()]
	param (	$Disk = "C:" )
	
	$LogicalDisk = Get-WMIObject Win32_LogicalDisk  -filter "DeviceID = '${Disk}:'"

    # get the drive index by finding the related Win32_DiskPartition instance:
	$DiskIndex = $LogicalDisk.psbase.GetRelated('Win32_DiskPartition') | Select-Object -ExpandProperty DiskIndex
		
	return $DiskIndex
}

## check that the drive exists
if ($SourceDriveLetter.Length -ne 1)
	{
		"Error: '$SourceDriveLetter' is not a valid drive letter."
		exit
	}
$SourcePath = "${SourceDriveLetter}:\"
if (!(test-path $SourcePath -pathtype Container))
	{
		"`nError: source drive path '$SourcePath' does not exist.`n"
		exit
	}

## define some internal variables
$starttime = $([DateTime]::Now)
$datetime = $starttime.ToString("DyyyyMMdd_THHmmss")
$BackupFileName = "Backup_${SourceDriveLetter}_${datetime}_${BackupType}.TBI"
$BackupFilePath = "$BackupFolder\$BackupFileName"
$LogFileName = "$BackupFolder\ifw.log"
$description = "IFW $BackupType backup on $datetime [locking $locking]"
$DriveIndex = Get-DriveIndex $SourceDriveLetter

## create the IFW backup file - uses VSS not PHYLOCK!
"Creating $BackupType backup file '$BackupFilePath'..."
"$BackupType backup for disk '$SourceDriveLetter' drive '$DriveIndex' - start time at $starttime"
$cmd = " ""$IFWPath\Image for Windows\V2\imagew.exe"" /b /d:$DriveIndex $locking $encryption $validation $compression $logging /logfile:""$LogFileName"" /f:""$BackupFilePath"" /desc:""$description"" "
"Running cmd $cmd"
cmd /c $cmd
$stoptime = $([DateTime]::Now)
$elaptime = $stoptime - $starttime
"$BackupType backup for disk '$SourceDriveLetter' drive '$DriveIndex' - stop time at $stoptime - elapsed time $elaptime)"

## do not start TBIView if imagew failed
if ($LastExitCode -ne 0)
{
	"`nIFW imagew.exe failed with exit code $LastExitCode.`n"
}
else
{
	## run TBIView on the new backup file
	cmd /c " ""$IFWPath\TBIView\tbiview.exe"" ""$BackupFilePath"" "
}

"IFW_FullBackup ended at $([DateTime]::Now). Hit ENTER to exit..."
read-host

## end of IFW_FullBackup.ps1
